## JS 部分
编码规范分为【强制】和【建议】两部分， 【强制】要求开发者必须遵从，代码检查中以error提示；
【建议】要求开发者自觉遵守，以保持统一的代码风格

#### 【建议】分号结尾，禁止多余分号
> good

```
const str = 'string';
```
> bad
```
const str = 'string';
const test = 'string'
```
#### 【建议】循环中更新子句的计数器朝着正确的方向移动
> good
```
for (var i = 0; i < 10; i++) {
}

```
> bad
```
for (var i = 0; i < 10; i--) {
}
for (var i = 10; i >= 0; i++) {
}
```
#### 【强制】缩进为2个空格，禁止使用tab
> good
```
function a (arg1, arg2) {
    // code
}
```
> bad
```
function a (arg1, arg2) {
  // code
}
```
####  【建议】getter 函数中要出现 return 语句
> good 
```
p = {
    get name(){
        return "nicholas";
    }
};

Object.defineProperty(p, "age", {
    get: function (){
        return 18;
    }
});

class P{
    get name(){
        return "nicholas";
    }
}

```
> bad
```
p = {
    get name(){
        // no returns.
    }
};

Object.defineProperty(p, "age", {
    get: function (){
        // no returns.
    }
});

class P{
    get name(){
        // no returns.
    }
}
```
#### 【强制】禁止 function 定义中出现重名参数
> good
```
function foo(a, b, c) {
    console.log(a, b, c);
}

var bar = function (a, b, c) {
    console.log(a, b, c);
};
```

> bad
```
function foo(a, b, a) {
    console.log("value of the second a:", a);
}

var bar = function (a, b, a) {
    console.log("value of the second a:", a);
};
```
#### 【强制】禁止不必要的布尔转换
> good
```
var foo = !!bar;
var foo = Boolean(bar);

function foo() {
    return !!bar;
}

var foo = bar ? !!baz : !!bat;
```
> bad
```
var foo = Boolean(!!bar);

var foo = new Boolean(!!bar);

if (!!foo) {
    // ...
}

if (Boolean(foo)) {
    // ...
}

while (!!foo) {
    // ...
}

do {
    // ...
} while (Boolean(foo));

for (; !!foo; ) {
    // ...
}
```
#### 【强制】要求调用 isNaN()检查 NaN 
> good
```
if (isNaN(foo)) {
    // ...
}

if (!isNaN(foo)) {
    // ...
}
```
> bad
```
if (foo == NaN) {
    // ...
}

if (foo != NaN) {
    // ...
}
```
#### 【建议】 禁止出现空代码块，但是允许出现空的catch子句
> good
```
try {
    doSomething();
} catch (ex) {
    
}
```
>bad
```
try {
   
} catch (ex) {
    
}
```
#### 【建议】限制圈复杂度，不超过3层
> good   

```
switch(x) {
    case 1: return x;
    case 2: x + '_test';
    case '': return 4;
    default: 
}
```
> bad
```

function a(x) {
    if (x === 1) {
        return x;
    } else if (x === 2) {
        return x + '_test';
    }  else if (x === '') {
        return 4;
    } else {
        
    }
}
```
#### 【建议】在点号之前换行
> good
```
var foo = object
.property;
var bar = object.property;
```
> bag
```
var foo = object.
property;
```
#### 【建议】数组元素换行：需要一致地使用数组元素之间的换行符

> good
```
var a = [];
var b = [1];
var c = [1, 2];
var d = [1, 2, 3];
var e = [
    1,
    2
];
var f = [
    1,
    2,
    3
];
var g = [
    function foo() {
        dosomething();
    }, function bar() {
        dosomething();
    }, function baz() {
        dosomething();
    }
];
var h = [
    function foo() {
        dosomething();
    },
    function bar() {
        dosomething();
    },
    function baz() {
        dosomething();
    }
];
```
> bad
```
var a = [
    1, 2,
    3
];
var b = [
    function foo() {
        dosomething();
    }, function bar() {
        dosomething();
    },
    function baz() {
        dosomething();
    }
];
```

#### 【强制】使用严格比较
强制在任何情况下都使用 === 和 !== 
> good
```
if (type === 1) {
    
}
if (type !== 1) {
    
}
```
> // bad
```
if (type == 1) {
    
}
if (type != 1) {
    
}
```
#### 【强制】标识符的最大和最小长度 (id-length)
限制最小长度不小于2，最大长度不超过20
> good
```
var num = 1;
var testNameLen = 11;
```
> bad 
```
var n = 1;
var testNameFormattingLenth = 23;
```
#### 【强制】强制行的最大长度 (max-len)
单行字符最大长度不超过120， tab字符宽度为4个字符
#### 【建议】强制文件的最大行数 (max-lines)
文件最大行数不超过1200行
#### 【建议】限制函数定义中最大参数个数(max-params)
函数定义中，参数最大长度不得超过5个   

开发者在个案的基础上考虑简化赋值的同时，应选择哪种风格更具可读性。
#### 【建议】如果有属性名称要求使用引号，则所有的属性名称都要使用引号。代码块一致，风格更美观
> good
```
var stuInfo = {
    name: '张三', 
    id: 123
};
```
> bad
```
var stuInfo = {
    'name': '张三', 
    id: 123
};
```
#### 【建议】使用一致的单引号 (quotes)
>good
```
var single = 'single';
var backtick = `back${x}tick`;
```
> bad
```
var double = "double";
var unescaped = "a string containing 'single' quotes";
```
#### 【建议】箭头函数的箭头之后要有空格 (arrow-spacing)
#### 【建议】 箭头函数参数加括号，箭头左右间隔一个空格单行函数体，函数体加大括号

> good

    (res) => res.name
    (res) => {
      return res.name
    }
> bad

    res=> res.name
    res =>{
      return res.name
    }
> good
```
(res) => res.name
  (res) => {
    return res.name
  }
```
> bad
```
res=> res.name
res =>{
  return res.name
}
```
#### 【建议】注释风格
- 文件和函数使用多行注释
- 文档注释前面要有一个空行
- 禁止使用行内注释，注释与描述间隔一个空格
- 避免使用 /\*...\*/这样的多行注释。有多行注释内容时，使用多个单行注释。 
> good
```
// str
const str = 'string';
// desc1
// desc2
let stuCount = 0;

/**
 * function a
 *
 * @param {String} arg1 first arg
 * @param {String=} arg2 second arg
 *
 * @return {String} return string value
 */
function a (arg1, arg2) {
    ...
    return ret;
}
```
####【建议】使用const和let定义变量，不使用var，const不能重新赋值，禁止重复声明变量。

>good
```
    const str = 'string';
    let num = 94;
```
>bad
```
    var num = 94;
    const str = 'stirng';
    str = 'str';
```
#### 【建议】禁用连续声明

>good
```
    let num1 = 94;
    let num2 = 95;
```
>bad
```
    let num1 = 94, num2 = 94;
```
#### 【建议】命名风格
- 文件命名：统一小写，适当使用分隔符
- 文件目录命名： 驼峰命名   


- 变量命名：驼峰命名

>good
``` 
    let isLoaded = 0;
```
>bad
```
    let is-loaded = 0;
    let isloaded = 0;
```
#### 【建议】参数变量禁止覆盖，使用本地变量处理

>good

    function b (arg1, arg2) {
      let _arg1 = arg1;
      const _arg2 = { ...arg2 };
      _arg1 = '';
      _arg2.name = '';
      ...
    }
>bad

    function b (arg1, arg2) {
      arg1 = '';
      arg2.name = '';
      ...
    }

## CSS 部分


### 【建议】需要用引号的地方，一律使用单引号
- 导入外部样式使用单引号

> good

    import '../style/common.less'
> bad

    import "../style/common.less"
* 字体定义加单引号

> good

    a {
      font-family: 'Times New Roman', 'Ancient Runes', serif;
    }
> bad

    a {
      font-family: "Times New Roman", Ancient Runes, serif;
    }
    
* 图片地址单引号

> good

    .div {
      background: url('http://...');
    }
> bad

    .div {
      background: url("http://...");
    }

    
#### 【建议】禁止单行定义样式

> good

    .btn {
      font-size: 12px;
    }
> bad

    .btn {font-size: 12px;}
    
#### 【建议】定义多个样式需换行

> good

    .btn,
    .text {
      font-size: 12px;
    }
> bad

    .btn, .text {
      font-size: 12px;
    }
    
#### 【建议】不同样式定义需换行，最大连续空行数1

> good

    .btn {
      color: #75c82b;
    }
    
    .text {
      font-size: 12px;
    }
> bad

    .btn {
      color: #75c82b;
    }
    
    
    .text {
      font-size: 12px;
    }
    .img {
      max-width: 100%;
      height: auto;
    }
    
### 属性

#### 【建议】属性冒号后间隔一个空格，分号结尾

> good

    .text {
      font-size: 0;
    }
> bad

    .text {
      font-size:0px
    }
    
####  【建议】0值不带单位

> good

    .text {
      font-size: 0;
    }
> bad

    .text {
      font-size: 0px;
    }
    
#### 【建议】嵌套定义内部块需要换行

> good

    .text {
      color: #222;
      
      .num {
        color: #ff193f;
      }
    }
> bad

    .text {
      color: #222;
      .num {
        color: #ff193f;
      }
    }
    
#### 【建议】缩进为2个空格，禁止使用tab

> good

    .text {
      color: #222;
      
      .num {
        color: #ff193f;
      }
    }
> bad

    .text {
    color: #222;
      
      .num {
          color: #ff193f;
      }
    }
    
### 色值

#### 【建议】颜色尽量使用16进制定义

> good

    .text {
      color: #fff;
    }
    .box {
      color: rgba(255, 255, 255, .5);
    }
> bad

    .text {
      color: rgba(255, 255, 255, 1);
    }
    
#### 【建议】色值定义全小写

> good

    .text {
      color: #fff;
    }
> bad

    .text {
      color: #fFf;
    }
    
#### 【建议】色值定义使用简写

> good

    .text {
      color: #fff;
    }
> bad

    .text {
      color: #ffffff;
    }