/**
 * Stylistic Issues Config
 * 本文件对Stylistic Issues规则进行自定制
 * 告警级别: 
 *  0 忽略问题，等同'off'
 *  1 给出警告，等同'warn'
 *  2 直接报错，等同'error'
 */
module.exports = {
  rules: {
    // 「关闭」 禁止标识符规则
    'id-blacklist': 0,
    // 「关闭」 禁用一元操作符 ++ 和 --规则
    'no-plusplus': 0,
    // 「关闭」 优先使用对象扩展而不是 Object.assign, 通常代码未支持此规则
    'prefer-object-spread': 0,
    // 「关闭」 禁止函数圆括号之前有一个空格, function 名称右侧与括号之间不得有空格
    'space-before-function-paren': 0,
    // 「关闭」 强制变量要么一起声明要么分开声明
    'one-var': 0,
    // 如果数组元素内或元素间有换行，则要求换行
    'array-bracket-newline': [1, { 'multiline': true }],
    // 数组元素见不必强制换行
    'array-element-newline': [1, 'consistent'],
    // 限制标识符的最小和最大长度
    'id-length': [1, { 'min': 2, 'max': 20 }],
    // 单行字符数(注释同)不得超过120
    'max-len': {
      'code': 120,
      'tabWidth': 4,
      'comments': true,
      'ignoreComments': false,
      'ignorePattern': true,
      'ignoreTrailingComments': false,
      'ignoreStrings': true,
      'ignoreTemplateLiterals': false,
      'ignoreRegExpLiterals': true
    },
    // 文件最大行数不得超过1000行，忽略空白行和只有注释内容的行
    'max-lines': {
      'max': 1000,
      'skipBlankLines': true,
      'skipComments': true
    },
    'max-params': {
      'max': 5
    },
    // 不建议拖尾逗号
    'comma-dangle': [1, {
      'arrays': 'never',
      'objects': 'never',
      'imports': 'never',
      'exports': 'never',
      'functions': 'never'
    }],
    // 如果有属性名称要求使用引号，则所有的属性名称都要使用引号。代码块一致，风格更美观
    'quote-props': [2, 'consistent-as-needed'],
    // 尽可能地使用单引号, 与es6`text ${变量名}`不冲突
    'quotes': [2, 'single'],
    // 「强制」分号应出现在句子末尾
    'semi-style': [2, 'last']
  }
};